@@ -19,6 +19,7 @@
 # is the function the new() command uses.
 
 from GTK import *
+import GDK    # make the GDK constants available
 import gtk; _gtk = gtk; del gtk
 import sys; _sys = sys; del sys
 import os ; _os  = os ; del os
@@ -46,6 +47,8 @@ class GtkObject:
 		if hasattr(v, '__class__') and v.__class__ in _supported:
 			v = v._o
 		_gtk.gtk_object_set(self._o, {key: v});
+	def __cmp__(self, other):
+		return cmp(self._o, other._o)
 	def flags(self, mask=None):
 		if mask:
 			return _gtk.GTK_OBJECT_FLAGS(self._o) & mask
@@ -55,7 +58,7 @@ class GtkObject:
 		_gtk.GTK_OBJECT_SET_FLAGS(self._o, flags)
 	def unset_flags(self, flags):
 		_gtk.GTK_OBJECT_UNSET_FLAGS(self._o, flags)
-	def set_data(self, key, val):
+	def set_data(self, key, value):
 		_gtk.gtk_object_set_data(self._o, key, value)
 	def get_data(self, key):
 		return _gtk.gtk_object_get_data(self._o, key)
@@ -141,6 +144,15 @@ class GtkWidget(GtkObject):
 		_gtk.gtk_widget_basic(self._o)
 	def destroy(self, obj=None):
 		_gtk.gtk_widget_destroy(self._o)
+	def dnd_drag_add(self, obj=None):
+		_gtk.gtk_widget_dnd_add(self._o)
+	def dnd_drag_set(self, enable, types):
+		_gtk.gtk_widget_dnd_drag_set(self._o, enable, types)
+	def dnd_drop_set(self, enable, types, destructive):
+		_gtk.gtk_widget_dnd_drop_set(self._o, enable, types,
+					     destructive)
+	def dnd_data_set(self, event, data):
+		_gtk.gtk_widget_dnd_data_set(self._o, event, data)
 	def draw(self, rect):
 		_gtk.gtk_widget_draw(self._o, rect)
 	def draw_default(self, obj=None):
@@ -171,6 +183,11 @@ class GtkWidget(GtkObject):
 		_gtk.gtk_widget_hide(self._o)
 	def hide_all(self, obj=None):
 		_gtk.gtk_widget_hide_all(self._o)
+	def hide_on_delete(self, obj=None):
+		return _gtk.gtk_widget_hide_on_delete(self._o)
+	def install_accelerator(self, at, sig, key, mods):
+		_gtk.gtk_widget_install_accelerator(self._o, at._at, sig, key,
+			mods)
 	def intersect(self, rect):
 		return _gtk.gtk_widget_intersect(self._o, rect)
 	def is_ancestor(self, a):
@@ -187,6 +204,8 @@ class GtkWidget(GtkObject):
 		_gtk.gtk_widget_queue_resize(self._o)
 	def realize(self, obj=None):
 		_gtk.gtk_widget_realize(self._o)
+	def remove_accelerator(self, at, sig):
+		_gtk.gtk_widget_remove_accelerator(self._o, at._o, sig)
 	def reparent(self, new_parent):
 		_gtk.gtk_widget_reparent(self._o, new_parent._o)
 	def restore_default_style(self):
@@ -211,6 +230,8 @@ class GtkWidget(GtkObject):
 		_gtk.gtk_widget_set_uposition(self._o, x, y)
 	def set_usize(self, w, h):
 		_gtk.gtk_widget_set_usize(self._o, w, h)
+	def shape_combine_mask(self, mask, ofs_x, ofs_y):
+	        _gtk.gtk_widget_shape_combine_mask(self._o,mask._o,ofs_x,ofs_y)
 	def show(self, obj=None):
 		_gtk.gtk_widget_show(self._o)
 	def show_all(self, obj=None):
@@ -301,6 +322,13 @@ class GtkToggleButton(GtkButton):
 			self._o = _gtk.gtk_toggle_button_new()
 		else:
 			self._o = _gtk.gtk_toggle_button_new_with_label(label)
+	def __getattr__(self, attr):
+	    attrs = {
+		    'active': _gtk.gtk_toggle_button_get_active
+	    }
+	    if attrs.has_key(attr):
+		    return attrs[attr](self._o)
+	    raise AttributeError, attr
 	def set_mode(self, di):
 		_gtk.gtk_toggle_button_set_mode(self._o, di)
 	def set_state(self, state):
@@ -405,6 +433,13 @@ class GtkCheckMenuItem(GtkMenuItem):
 			self._o = _gtk.gtk_check_menu_item_new()
 		else:
 			self._o =_gtk.gtk_check_menu_item_new_with_label(label)
+	def __getattr__(self, attr):
+		attrs = {
+			'active': _gtk.gtk_check_menu_item_get_active
+		}
+		if attrs.has_key(attr):
+			return attrs[attr](self._o)
+		raise AttributeError, attr
 	def set_show_toggle(self, always):
 		_gtk.gtk_check_menu_item_set_show_toggle(self._o, always)
 	def set_state(self, state):
@@ -722,10 +757,14 @@ class GtkCList(GtkContainer):
 		_gtk.gtk_clist_column_titles_passive(self._o)
 	def column_titles_show(self, obj=None):
 		_gtk.gtk_clist_column_titles_show(self._o)
+	def find_row_from_data(self, data):
+		return _gtk.gtk_clist_find_row_from_data(self._o, data)
 	def freeze(self, obj=None):
 		_gtk.gtk_clist_freeze(self._o)
 	def get_cell_type(self, r, c):
 		return _gtk.gtk_clist_get_cell_type(self._o, r, c)
+	def get_row_data(self, row):
+		return _gtk.gtk_clist_get_row_data(self._o, row)
 	def get_text(self, r, c):
 		return _gtk.gtk_clist_get_text(self._o, r, c)
 	def insert(self, row, values):
@@ -752,6 +791,8 @@ class GtkCList(GtkContainer):
 		_gtk.gtk_clist_set_foreground(self._o, row, colour)
 	def set_policy(self, vsp, hsp):
 		_gtk.gtk_clist_set_policy(self._o, vsp, hsp)
+	def set_row_data(self, row, data):
+		_gtk.gtk_clist_set_row_data(self._o, row, data)
 	def set_row_height(self, height):
 		_gtk.gtk_clist_set_row_height(self._o, height)
 	def set_shift(self, row, col, v, h):
@@ -1125,6 +1166,10 @@ class GtkSpinButton(GtkEntry):
 		_gtk.gtk_spin_button_set_value(self._o, val)
 	def set_update_policy(self, pol):
 		_gtk.gtk_spin_button_set_update_policy(self._o, pol)
+	def set_numeric(self, numeric):
+		_gtk.gtk_spin_button_set_numeric(self._o, numeric)
+	def spin(self, direction, step):
+		_gtk.gtk_spin_button_spin(self._o, direction, step)
 
 class GtkText(GtkEditable):
 	def __init__(self, hadj=None, vadj=None, _obj=None):
@@ -1256,22 +1301,6 @@ class GtkRange(GtkWidget):
 		_gtk.gtk_range_draw_step_forw(self._o)
 	def draw_step_back(self, obj=None):
 		_gtk.gtk_range_draw_step_back(self._o)
-	def trough_click(self, x, y):
-		_gtk.gtk_range_trough_click(self._o, x, y)
-	def default_hslider_update(self, obj=None):
-		_gtk.gtk_range_default_hslider_update(self._o)
-	def default_vslider_update(self, obj=None):
-		_gtk.gtk_range_default_vslider_update(self._o)
-	def default_htrough_click(self, x, y):
-		_gtk.gtk_range_default_htrough_click(self._o, x, y)
-	def default_vtrough_click(self, x, y):
-		_gtk.gtk_range_default_vtrough_click(self._o, x, y)
-	def default_hmotion(self, dx, dy):
-		_gtk.gtk_range_default_hmotion(self._o, dx, dy)
-	def default_vmotion(self, dx, dy):
-		_gtk.gtk_range_default_vmotion(self._o, dx, dy)
-	def calc_value(self, pos):
-		return _gtk.gtk_range_calc_value(self._o, pos)
 
 class GtkScale(GtkRange):
 	def __init__(self, _obj=None):
@@ -1415,25 +1444,73 @@ def new(tp, props={}):
 	_filtprops(props)
 	return _obj2inst(_gtk.gtk_object_new(tp, props))
 
+# flow control
 def mainloop():
 	_gtk.gtk_main()
 def mainquit(*args):
 	_gtk.gtk_main_quit()
 def mainiteration(block=TRUE):
 	_gtk.gtk_main_iteration(block)
+
+# idle/quit/timeout handlers
 def idle_add(callback):
 	return _gtk.gtk_idle_add(callback)
+def idle_add_priority(priority, callback):
+	return _gtk.gtk_idle_add_priority(priority, callback)
 def idle_remove(tag):
 	_gtk.gtk_idle_remove(tag)
+def quit_add(mainlevel, callback):
+        return _gtk.gtk_quit_add(mainlevel, callback)
+def quit_add_destroy(mainlevel, object):
+	_gtk.gtk_quit_add_destroy(mainlevel, object._o)
+def quit_remove(tag):
+	_gtk.gtk_quit_remove(tag)
 def timeout_add(timeout, callback):
 	return _gtk.gtk_timeout_add(timeout, callback)
 def timeout_remove(tag):
 	_gtk.gtk_timeout_remove(tag)
+
+# input (file handle) handler
+def input_add(source, condition, callback):
+	if hasattr(source, 'fileno'):
+		# handle python file handles
+		def wrapper(source, condition, real_s=source,real_cb=callback):
+			real_cb(real_s, condition)
+		return _gtk.gtk_input_add(source.fileno(), condition, wrapper)
+	else:
+		return _gtk.gtk_input_add(source, condition, callback)
+def input_remove(tag):
+	_gtk.gtk_input_remove(tag)
+
+# these functions govern modality
+def grab_add(wid):
+	_gtk.gtk_grab_add(wid._o)
+def grab_remove(wid):
+	_gtk.gtk_grab_remove(wid._o)
+
+# RC parsing
 def rc_parse(fname):
 	_gtk.gtk_rc_parse(fname)
 def rc_parse_string(string):
 	_gtk.gtk_rc_parse_string(string)
+
+# font loading
 def load_font(font):
 	return _gtk.gdk_font_load(font)
 def load_fontset(fontset):
 	return _gtk.gdk_fondset_load(fontset)
+
+# colour allocation
+def colour_alloc(parent, red, green=None, blue=None):
+	if green == None and blue == None:
+		green, blue = divmod(red, 256)
+		red, green  = divmod(green, 256)
+		red = red % 256
+
+		red   = (red   + 1) * 256 - 1
+		green = (green + 1) * 256 - 1
+		blue  = (blue  + 1) * 256 - 1
+	if hasattr(parent, '_o'): parent = parent._o
+	return _gtk.gdk_color_alloc(parent, red, green, blue)
+# This one's for the Americans...
+color_alloc = colour_alloc
