@@ -105,13 +105,14 @@ class gtkoutfile:
 	def tell(self):      raise IOError, (29, 'Illegal seek')
 	truncate = tell
 
-class Console(GtkWindow):
-	def __init__(self, namespace={}, copyright='', title='Python', menu=None):
-	        GtkWindow.__init__(self)
-		self.set_title(title)
-		self.set_usize(475, 300)
-		self.connect("destroy", self.quit)
-		self.connect("delete_event", self.quit)
+class Console(GtkVBox):
+	def __init__(self, namespace={}, copyright='', quit_cb=None):
+		GtkVBox.__init__(self, spacing=2)
+		self.border_width(2)
+		self.copyright = copyright
+		#self.set_usize(475, 300)
+
+		self.quit_cb = quit_cb
 
 		#load the fonts we will use
 		self.normal = load_font(
@@ -123,26 +124,15 @@ class Console(GtkWindow):
 		self.command = load_font(
 			"-*-helvetica-bold-r-normal-*-*-100-*-*-*-*-*-*")
 
-		self.main_vbox = GtkVBox(spacing=4)
-		self.add(self.main_vbox)
-		self.main_vbox.show()
-
-		if menu:
-		    self.main_vbox.pack_start(menu, fill=TRUE, expand=FALSE)
-
 		self.inp = GtkHBox()
-		self.main_vbox.pack_start(self.inp, padding=4)
+		self.pack_start(self.inp)
 		self.inp.show()
 
 		self.text = GtkText()
 		self.text.unset_flags(CAN_FOCUS)
-		self.text.style = self.text.get_style()
-		self.text.fg = self.text.style.fg[STATE_NORMAL]
-		self.text.bg = self.text.style.white
 		self.text.set_editable(FALSE)
 		self.text.set_usize(500, 400)
 		self.inp.pack_start(self.text, padding=1)
-		self.text.realize()
 		self.text.show()
 
 		self.vscroll = GtkVScrollbar(self.text.get_vadjustment())
@@ -151,16 +141,13 @@ class Console(GtkWindow):
 		self.inp.pack_end(self.vscroll, expand=FALSE)
 		self.vscroll.show()
 
-		self.hdlbox = GtkHandleBox()
-		self.main_vbox.pack_end(self.hdlbox, expand=FALSE, padding=4)
-		self.hdlbox.show()
-
-		self.inputbox = GtkHBox(spacing=4)
-		self.hdlbox.add(self.inputbox)
+		self.inputbox = GtkHBox(spacing=2)
+		self.pack_end(self.inputbox, expand=FALSE)
 		self.inputbox.show()
 
 		self.prompt = GtkLabel(sys.ps1)
-		self.prompt.set_usize(22, -1)
+		self.prompt.set_padding(xp=2, yp=0)
+		self.prompt.set_usize(26, -1)
 		self.inputbox.pack_start(self.prompt, fill=FALSE, expand=FALSE)
 		self.prompt.show()
 
@@ -176,19 +163,10 @@ class Console(GtkWindow):
 		self.inputbox.pack_start(self.line, padding=2)
 		self.line.show()
 
-		self.line.grab_focus()
-
 		# now let the text box be resized
 		self.text.set_usize(0, 0)
 		self.line.set_usize(0, -1)
 
-		self.text.insert(self.title, self.text.fg,
-				 self.text.bg, 'Python %s\n%s\n\n' %
-				 (sys.version, sys.copyright) +
-				 'Interactive Python-GTK Console - ' +
-				 'Copyright (C) 1998 James Henstridge\n\n' +
-				 copyright + '\n')
-
 		self.namespace = namespace
 
 		self.cmd = ''
@@ -205,41 +183,49 @@ class Console(GtkWindow):
 		self.histpos = 0
 		self.namespace['__history__'] = self.history
 
+	def init(self):
+		self.text.realize()
+		self.text.style = self.text.get_style()
+		self.text.fg = self.text.style.fg[STATE_NORMAL]
+		self.text.bg = self.text.style.white
+
+		self.text.insert(self.title, self.text.fg,
+				 self.text.bg, 'Python %s\n%s\n\n' %
+				 (sys.version, sys.copyright) +
+				 'Interactive Python-GTK Console - ' +
+				 'Copyright (C) 1998 James Henstridge\n\n' +
+				 self.copyright + '\n')
+		self.line.grab_focus()
+
 	def quit(self, *args):
 		self.hide()
 		self.destroy()
-		mainquit()
+		if self.quit_cb: self.quit_cb()
 
 	def key_function(self, entry, event):
-		# these chould be defined somewhere else ...
-		GDK_Return = 0xFF0D
-		GDK_KP_Up = 0xFF97
-		GDK_Up = 0xFF52
-		GDK_KP_Down = 0xFF99
-		GDK_Down = 0xFF54
-		GDK_D = 0x044
-		GDK_d = 0x064
-		GDK_CONTROL_MASK = 1 << 2
-		if event.keyval == GDK_Return:
+		if event.keyval == GDK.Return:
 			self.line.emit_stop_by_name("key_press_event")
 			self.eval()
-		elif event.keyval in (GDK_KP_Up, GDK_Up):
+		elif event.keyval in (GDK.KP_Up, GDK.Up):
 			self.line.emit_stop_by_name("key_press_event")
 			self.historyUp()
-			#self.closer.grab_focus()
-		elif event.keyval in (GDK_KP_Down, GDK_Down):
+			idle_add(self.focus_text)
+		elif event.keyval in (GDK.KP_Down, GDK.Down):
 			self.line.emit_stop_by_name("key_press_event")
 			self.historyDown()
-			#self.closer.grab_focus()
-		elif event.keyval in (GDK_D, GDK_d) and \
-		     event.state & GDK_CONTROL_MASK:
+			idle_add(self.focus_text)
+		elif event.keyval in (GDK.D, GDK.d) and \
+		     event.state & GDK.CONTROL_MASK:
 			self.line.emit_stop_by_name("key_press_event")
 			self.ctrld()
 
-	def ctrld(self, *args):
+	def focus_text(self):
+		self.line.grab_focus()
+
+	def ctrld(self):
 		self.quit()
 
-	def historyUp(self, *args):
+	def historyUp(self):
 		if self.histpos > 0:
 			l = self.line.get_text()
 			if len(l) > 0 and l[0] == '\n': l = l[1:]
@@ -248,7 +234,7 @@ class Console(GtkWindow):
 			self.histpos = self.histpos - 1
 			self.line.set_text(self.history[self.histpos])
 			
-	def historyDown(self, *args):
+	def historyDown(self):
 		if self.histpos < len(self.history) - 1:
 			l = self.line.get_text()
 			if len(l) > 0 and l[0] == '\n': l = l[1:]
@@ -257,7 +243,7 @@ class Console(GtkWindow):
 			self.histpos = self.histpos + 1
 			self.line.set_text(self.history[self.histpos])
 
-	def eval(self, *args):
+	def eval(self):
 		l = self.line.get_text() + '\n'
 		if len(l) > 1 and l[0] == '\n': l = l[1:]
 		self.histpos = len(self.history) - 1
@@ -283,7 +269,7 @@ class Console(GtkWindow):
 		if not bracketsBalanced(self.cmd2) or l[-1] == ':' or \
 				l[-1] == '\\' or l[0] in ' \11':
 			self.prompt.set(sys.ps2)
-			self.hdlbox.queue_draw()
+			self.prompt.queue_draw()
 			return
 		self.run(self.cmd)
 		self.cmd = ''
@@ -306,16 +292,32 @@ class Console(GtkWindow):
 			else:
 				traceback.print_exc()
 		self.prompt.set(sys.ps1)
-		self.hdlbox.queue_draw()
+		self.prompt.queue_draw()
 		adj = self.text.get_vadjustment()
 		adj.set_value(adj.upper - adj.page_size)
 		sys.stdout, self.stdout = self.stdout, sys.stdout
 		sys.stderr, self.stderr = self.stderr, sys.stderr
 
 def gtk_console(ns, title='Python', copyright='', menu=None):
-	cons = Console(title=title, namespace=ns, copyright=copyright,
-		       menu=menu)
+	win = GtkWindow()
+	win.set_usize(475, 300)
+	win.connect("destroy", mainquit)
+	win.connect("delete_event", mainquit)
+	win.set_title(title)
+	cons = Console(namespace=ns, copyright=copyright, quit_cb=mainquit)
+	if menu:
+		box = GtkVBox()
+		win.add(box)
+		box.show()
+		box.pack_start(menu, expand=FALSE)
+		menu.show()
+		box.pack_start(cons)
+	else:
+		win.add(cons)
 	cons.show()
+	win.show()
+	win.set_usize(0,0)
+	cons.init()
 	mainloop()
 
 if __name__ == '__main__':
