@@ -32,7 +32,7 @@ button = new(GtkButton, {
 	'GtkWidget::visible': 1
 })
 
-tt.set_tips(button, 'Prints "Hello World"')
+tt.set_tip(button, 'Prints "Hello World"', '')
 tt.enable()
 
 window.show()
