@@ -1,6 +1,7 @@
 #!/usr/bin/python
 
 from Gtkinter import *
+import GtkExtra
 
 wins = {}
 
@@ -741,6 +742,9 @@ def create_clist(_button):
 		box2.show()
 
 		clist = GtkCList(7, map(lambda i: "Title "+str(i), range(7)))
+		clist.set_column_width(0, 100)
+		for i in range(1, 7):
+			clist.set_column_width(i, 80)
 
 		text = map(lambda i: "Column "+str(i), range(7))
 		text[1] = "Right"
@@ -1173,6 +1177,85 @@ def create_panes(_button):
 		button.show()
 	wins["panes"].show()
 
+def create_dnd(_button):
+	if not wins.has_key("dnd"):
+		# note that these two functions don't use any global variables
+		# to communicate.  In fact, try openning two copies of
+		# testgtk.py (or the C version) and drag between them.
+		def dnd_drop(button, event):
+			msg = "Drop data of type %s was:\n\n%s" % \
+				(event.data_type, event.data)
+			GtkExtra.message_box("Drop", msg,
+				("Continue with life in\n" +
+				"spite of this oppression",))
+		def dnd_drag_request(button, event):
+			dnd_string = "Bill Gates demands royalties for\n" + \
+				     "your use of his innovation."
+			button.dnd_data_set(event, dnd_string)
+		win = GtkWindow(WINDOW_TOPLEVEL)
+		wins["dnd"] = win
+		win.connect("destroy", win.hide)
+		win.connect("delete_event", lambda win,_e: win.hide())
+		win.set_title("Drag -N- Drop")
+
+		box1 = GtkVBox()
+		win.add(box1)
+		box1.show()
+
+		box2 = GtkHBox(spacing=5)
+		box2.border_width(10)
+		box1.pack_start(box2)
+		box2.show()
+
+		frame = GtkFrame("Drag")
+		box2.pack_start(frame)
+		frame.show()
+
+		box3 = GtkVBox(spacing=5)
+		box3.border_width(5)
+		frame.add(box3)
+		box3.show()
+
+		button = GtkButton("Drag me!")
+		box3.pack_start(button)
+		button.show()
+		button.realize()
+		button.connect("drag_request_event", dnd_drag_request)
+		button.dnd_drag_set(TRUE, ['text/plain'])
+
+		frame = GtkFrame("Drop")
+		box2.pack_start(frame)
+		frame.show()
+
+		box3 = GtkVBox(spacing=5)
+		box3.border_width(5)
+		frame.add(box3)
+		box3.show()
+
+		button = GtkButton("To")
+		box3.pack_start(button)
+		button.show()
+		button.realize()
+		button.connect("drop_data_available_event", dnd_drop)
+		button.dnd_drop_set(TRUE, ['text/plain'], FALSE)
+
+		separator = GtkHSeparator()
+		box1.pack_start(separator, expand=FALSE)
+		separator.show()
+
+		box2 = GtkVBox(spacing=10)
+		box2.border_width(10)
+		box1.pack_start(box2, expand=FALSE)
+		box2.show()
+
+		button = GtkButton("close")
+		button.connect("clicked", win.hide)
+		box2.pack_start(button)
+		button.set_flags(CAN_DEFAULT)
+		button.grab_default()
+		button.show()
+	wins['dnd'].show()
+
 def create_progress_bar(_button):
 	if not wins.has_key("progress_bar"):
 		win = GtkDialog()
@@ -1194,16 +1277,14 @@ def create_progress_bar(_button):
 		vbox.pack_start(pbar)
 		pbar.show()
 
-		func = [None, None]
-		def timeout(pbar=pbar, pcnt=[0], func=func):
+		def timeout(pbar=pbar, pcnt=[0]):
 			pcnt[0] = (pcnt[0] + 0.02) % 1.0
 			pbar.update(pcnt[0])
-			func[1] = timeout_add(100, func[0])
+			return TRUE
 
-		func[0] = timeout
-		win.func = func
+		win.func = [timeout, 0]
 
-		def killit(arg1=None, arg2=None, tid=func, win=win):
+		def killit(arg1=None, arg2=None, tid=win.func, win=win):
 			timeout_remove(tid[1])
 			win.hide()
 		win.connect("destroy", killit)
@@ -1248,12 +1329,12 @@ def create_timeout_test(_button):
 		label.show()
 
 		func = [None, 0]
-		def timeout_test(label=label, count=[0], func=func):
+		def timeout_test(label=label, count=[0]):
 			count[0] = count[0] + 1
 			label.set("count: " + str(count[0]))
-			func[1] = timeout_add(100, func[0])
+			return TRUE
 
-		func[0] = timeout_test
+		func = [timeout_test, 0]
 		def start_timeout_test(_button, func=func):
 			if not func[1]:
 				func[1] = timeout_add(100, func[0])
@@ -1265,6 +1346,8 @@ def create_timeout_test(_button):
 		button = GtkButton("close")
 		button.connect("clicked", win.hide)
 		win.action_area.pack_start(button)
+		button.set_flags(CAN_DEFAULT)
+		button.grab_default()
 		button.show()
 
 		button = GtkButton("start")
@@ -1275,8 +1358,6 @@ def create_timeout_test(_button):
 		button = GtkButton("stop")
 		button.connect("clicked", stop_timeout_test)
 		win.action_area.pack_start(button)
-		button.set_flags(CAN_DEFAULT)
-		button.grab_default()
 		button.show()
 	wins["timeout_test"].show()
 
@@ -1293,17 +1374,16 @@ def create_idle_test(_button):
 		win.vbox.pack_start(label)
 		label.show()
 
-		func = [None, 0]
-		def idle_test(label=label, count=[0], func=func):
+		def idle_test(label=label, count=[0]):
 			count[0] = count[0] + 1
 			label.set("count: " + str(count[0]))
-			func[1] = idle_add(func[0])
+			return TRUE
 
-		func[0] = idle_test
-		def start_idle_test(_button, func=func):
+		win.func = [idle_test, 0]
+		def start_idle_test(_button, func=win.func):
 			if not func[1]:
 				func[1] = idle_add(func[0])
-		def stop_idle_test(_button, func=func):
+		def stop_idle_test(_button, func=win.func):
 			if func[1]:
 				idle_remove(func[1])
 				func[1] = 0
@@ -1311,6 +1391,8 @@ def create_idle_test(_button):
 		button = GtkButton("close")
 		button.connect("clicked", win.hide)
 		win.action_area.pack_start(button)
+		button.set_flags(CAN_DEFAULT)
+		button.grab_default()
 		button.show()
 
 		button = GtkButton("start")
@@ -1321,8 +1403,6 @@ def create_idle_test(_button):
 		button = GtkButton("stop")
 		button.connect("clicked", stop_idle_test)
 		win.action_area.pack_start(button)
-		button.set_flags(CAN_DEFAULT)
-		button.grab_default()
 		button.show()
 	wins["idle_test"].show()
 
@@ -1357,7 +1437,7 @@ def create_main_window():
 		"notebook": create_notebook,
 		"panes": create_panes,
 		"shapes": None, #create_shapes,
-		"dnd": None, #create_dnd,
+		"dnd": create_dnd,
 		"progress bar": create_progress_bar,
 		"preview color": None, #create_color_preview,
 		"preview gray": None, #create_gray_preview,
@@ -1369,7 +1449,7 @@ def create_main_window():
 		"test": None, #create_test
 	}
 	win = GtkWindow()
-	#win.set_name("main window")
+	win.set_name("main window")
 	win.set_usize(200, 400)
 	win.set_uposition(20, 20)
 	win.connect("destroy", mainquit)
