@@ -5,14 +5,17 @@ import string
 objects = ( "Adjustment", "Alignment", "Arrow", "Bin", "Box", "Button",
   "ButtonBox", "CheckButton", "CheckMenuItem", "CList", "ColorSelection",
   "ColorSelectionDialog", "Combo", "Container", "Curve", "Data", "Dialog",
-  "DrawingArea", "Entry", "EventBox", "FileSelection", "Fixed", "Frame",
+  "DrawingArea", "Editable", "Entry", "EventBox", "FileSelection", "Fixed",
+  "Frame",
   "GammaCurve", "HandleBox", "HBox", "HButtonBox", "HPaned", "HRuler",
   "HScale", "HScrollbar", "HSeparator", "Image", "InputDialog", "Item",
   "Label", "List", "ListItem", "Menu", "MenuBar", "MenuItem", "MenuShell",
   "Misc", "Notebook", "Object", "OptionMenu", "Paned", "Pixmap", "Preview",
   "ProgressBar", "RadioButton", "RadioMenuItem", "Range", "Ruler", "Scale",
-  "Scrollbar", "ScrolledWindow", "Separator", "Statusbar", "Table", "Text",
-  "ToggleButton", "Toolbar", "Tree", "TreeItem", "VBox", "VButtonBox",
+  "Scrollbar", "ScrolledWindow", "Separator", "SpinButton", "Statusbar",
+  "Table", "Text",
+  "ToggleButton", "Toolbar", "Tooltips", "Tree", "TreeItem", "VBox",
+  "VButtonBox",
   "Viewport", "VPaned", "VRuler", "VScale", "VScrollbar", "VSeparator",
   "Widget", "Window" )
 
